/******************************************************************************
 * program:     rasimg library                                                *
 * function:    Mask filtering image.                                         *
 * modul:       mask.cc                                                       *
 * licency:     GPL or LGPL                                                   *
 ******************************************************************************/
#include <stddef.h>
#include <stdio.h>
//#include <io.h>

#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#include "typedfs.h"
#include "raster.h"
#include "ras_prot.h"
#include "imageop.h"

#ifndef __UNIX__
// #include <io.h>
#endif

#include <matrix.h>


#ifndef __Have_stricmp
 #ifdef __Have_strcasecmp
  #define stricmp strcasecmp
  #define strnicmp strncasecmp
 #endif
 #ifndef stricmp        //already fixed?
  #define stricmp strcmp
 #endif
#endif


//#ifdef RasImg
//#include <alloc.h>

const char *GetExtension(const char *FullName)
{
const char *NameStep;
int LastPos;

if(FullName==NULL) return(NULL);

LastPos=strlen(FullName);
NameStep=FullName+LastPos+1;
while(NameStep>FullName)
	{
	NameStep--;

	if(*NameStep=='/' || *NameStep=='\\' || *NameStep==':')
		break;
	if(*NameStep=='.')
		return(NameStep);
	}

return(FullName+LastPos);
}

////////////////////////////////////////////////////////////////////


char Verbosity = 0;

void PrintHeader(void)
{
char HdrPrinted=0;

  if(Verbosity>=0 && HdrPrinted==0)
    {
    puts("\n\r <<< MASK >>> Mask picture filtering (c)1997-2022 F&TSoft ");
    HdrPrinted = 1;
    }
}


/////////////////////Main CODE starts here/////////////////////////////////


int main(int argc,char *argv[])
{
Image I1,I2;
const char *InputImg=NULL, *OutputImg=NULL;
const char *s;
TImageFileHandler *LoadPicture_XXX;
int i;
int MaskSize = 3;
bool Conv2Gray = false;
int Error = 0;

	/* Parse Arguments */
for(i=1;i<argc;i++)
  {
  s=argv[i];
  if(s==NULL) continue;
  if( *s==0 ) continue;

  if(*s=='-' || *s=='/')
    {
    if(strcmp(s+1,"help")==0 || strcmp(s+1,"-help")==0 || strcmp(s+1,"?")==0)
      {
        PrintHeader();
        printf("\n");
        for(LoadPicture_XXX=TImageFileHandler::first();LoadPicture_XXX!=NULL;LoadPicture_XXX=LoadPicture_XXX->next())
	  {
	  printf("%s %s%s; ",LoadPicture_XXX->extension(),
			     LoadPicture_XXX->LoadPicture==NULL?"":"R",
			     LoadPicture_XXX->SavePicture==NULL?"":"W");
	  }
	printf(//"\n/NoScale      - means truncating image data"
	       "\n/Gray         - discards all color informations"
	       //"\n/SEPARATE[R|G|B] - remove one channel from true color images"
	       "\n/MaskSize n  - cut only plane with number n"
	       "\n/s            - decrease verbosity level"
	       "\n");
        return(0);
      }
    if(strcmp(s+1,"MaskSize") == 0)
      {
      MaskSize = atoi(argv[++i]);
      continue;
      }
    if(strcmp(s+1,"s") == 0)
      {
      Verbosity = -1;
      continue;
      }
     if(strcmp(s+1,"Gray") == 0)
     {
       Conv2Gray = true;
       continue;
     }
    }

  if(InputImg==NULL)
     InputImg=s;
  else
     if(OutputImg==NULL) OutputImg=s;
  }
	/* End of reading arguments */

  PrintHeader();
  if(Verbosity>=0) printf("\rLoading");
  //MarkTime=GetInterval();

  //for(i=1;i<=10;i++)
  I1 = LoadPicture(InputImg);
  if(I1.Raster==NULL && I1.VecImage==NULL)
	{
	printf("\n\rCannot load input picture:\"%s\" Error %d",InputImg,Error);
	goto End;
	}

  if(Conv2Gray)
  {
    if(Verbosity>=0) printf("\rConverting to gray ");
    Convert2Gray(I1);
  }

  //MarkTime=GetInterval();

  if(Verbosity>=0) printf("\rMask filtering");
  I1 = Mask(I1,MaskSize);

  if(Verbosity>=0) printf("\rSaving        ");

  Error = SavePicture(OutputImg,I1);
  if(Error)
    {
    printf("\n\rCannot save output picture:\"%s\" Error %d",OutputImg,Error);
    }
  else
    {
    if(Verbosity>=0)
      {
      //printf("time : %f",abs(GetInterval()-MarkTime)/18.2);
      puts("\rOK     ");
      }
    }

End:
  return(Error);
}
